@@ -43,6 +43,7 @@ static const char* cmCTestErrorMatches[] = {
   "^[Bb]us [Ee]rror",
   "^[Ss]egmentation [Vv]iolation",
   "^[Ss]egmentation [Ff]ault",
+  ":.*[Pp]ermission [Dd]enied",
   "([^ :]+):([0-9]+): ([^ \\t])",
   "([^:]+): error[ \\t]*[0-9]+[ \\t]*:",
   "^Error ([0-9]+):",
@@ -61,11 +62,11 @@ static const char* cmCTestErrorMatches[] = {
   ": syntax error ",
   "^collect2: ld returned 1 exit status",
   "ld terminated with signal",
-  "Unsatisfied symbols:",
+  "Unsatisfied symbol",
   "^Unresolved:",
-  "Undefined symbols:",
+  "Undefined symbol",
   "^Undefined[ \\t]+first referenced",
-  "^CMake Error:",
+  "^CMake Error.*:",
   ":[ \\t]cannot find",
   ":[ \\t]can't find",
   ": \\*\\*\\* No rule to make target \\`.*\\'.  Stop",
@@ -129,6 +130,7 @@ static const char* cmCTestWarningMatches[] = {
   "\\([0-9]*\\): remark #[0-9]*",
   "\".*\", line [0-9]+: remark\\([0-9]*\\):",
   "cc-[0-9]* CC: REMARK File = .*, Line = [0-9]*",
+  "^CMake Warning.*:",
   0
 };
 
@@ -174,8 +176,8 @@ cmCTestWarningErrorFileLine[] = {
 //----------------------------------------------------------------------
 cmCTestBuildHandler::cmCTestBuildHandler()
 {
-  this->MaxPreContext = 6;
-  this->MaxPostContext = 6;
+  this->MaxPreContext = 10;
+  this->MaxPostContext = 10;
 
   this->MaxErrors = 50;
   this->MaxWarnings = 50;
@@ -214,8 +216,8 @@ void cmCTestBuildHandler::Initialize()
   this->ErrorsAndWarnings.clear();
   this->LastErrorOrWarning = this->ErrorsAndWarnings.end();
   this->PostContextCount = 0;
-  this->MaxPreContext = 6;
-  this->MaxPostContext = 6;
+  this->MaxPreContext = 10;
+  this->MaxPostContext = 10;
   this->PreContext.clear();
 
   this->TotalErrors = 0;
@@ -249,6 +251,20 @@ void cmCTestBuildHandler::PopulateCustomVectors(cmMakefile *mf)
                              "CTEST_CUSTOM_MAXIMUM_NUMBER_OF_WARNINGS",
                              this->MaxWarnings);
 
+  int n = -1;
+  this->CTest->PopulateCustomInteger(mf, "CTEST_CUSTOM_ERROR_PRE_CONTEXT", n);
+  if (n != -1)
+    {
+    this->MaxPreContext = static_cast<size_t>(n);
+    }
+
+  n = -1;
+  this->CTest->PopulateCustomInteger(mf, "CTEST_CUSTOM_ERROR_POST_CONTEXT", n);
+  if (n != -1)
+    {
+    this->MaxPostContext = static_cast<size_t>(n);
+    }
+
   // Record the user-specified custom warning rules.
   if(const char* customWarningMatchers =
      mf->GetDefinition("CTEST_CUSTOM_WARNING_MATCH"))
@@ -551,6 +567,7 @@ class cmCTestBuildHandler::FragmentCompare
 {
 public:
   FragmentCompare(cmFileTimeComparison* ftc): FTC(ftc) {}
+  FragmentCompare(): FTC(0) {}
   bool operator()(std::string const& l, std::string const& r)
     {
     // Order files by modification time.  Use lexicographic order
@@ -958,7 +975,7 @@ int cmCTestBuildHandler::RunMakeCommand(const char* command,
   this->ProcessBuffer(0, 0, tick, tick_len, ofs,
     &this->BuildProcessingErrorQueue);
   cmCTestLog(this->CTest, OUTPUT, " Size of output: "
-    << int(this->BuildOutputLogSize / 1024.0) << "K" << std::endl);
+    << ((this->BuildOutputLogSize + 512) / 1024) << "K" << std::endl);
 
   // Properly handle output of the build command
   cmsysProcess_WaitForExit(cp, 0);
@@ -1170,7 +1187,7 @@ void cmCTestBuildHandler::ProcessBuffer(const char* data, int length,
     if ( tick % tick_line_len == 0 && tick > 0 )
       {
       cmCTestLog(this->CTest, HANDLER_OUTPUT, "  Size: "
-        << int((this->BuildOutputLogSize / 1024.0) + 1) << "K" << std::endl
+        << ((this->BuildOutputLogSize + 512) / 1024) << "K" << std::endl
         << "    ");
       }
     }
