@@ -46,12 +46,6 @@ public:
   static void ExpandRegistryValues(std::string& source,
                                    KeyWOW64 view = KeyWOW64_Default);
 
-  /**
-   * Platform independent escape spaces, unix uses backslash,
-   * windows double quotes the string.
-   */
-  static std::string EscapeSpaces(const char* str);
-
   ///! Escape quotes in a string.
   static std::string EscapeQuotes(const char* str);
 
@@ -109,6 +103,12 @@ public:
       cmSystemTools::s_ErrorOccured = false;
     }
   
+  /**
+   * Does a string indicates that CMake/CPack/CTest internally
+   * forced this value. This is not the same as On, but this
+   * may be considered as "internally switched on".
+   */
+  static bool IsInternallyOn(const char* val);
   /** 
    * does a string indicate a true or on value ? This is not the same
    * as ifdef. 
@@ -379,13 +379,11 @@ public:
 
   /** Create tar */
   static bool ListTar(const char* outFileName,
-                      std::vector<cmStdString>& files, 
                       bool gzip, bool verbose);
   static bool CreateTar(const char* outFileName,
                         const std::vector<cmStdString>& files, bool gzip,
-                        bool verbose);
-  static bool ExtractTar(const char* inFileName,
-                         const std::vector<cmStdString>& files, bool gzip, 
+                        bool bzip2, bool verbose);
+  static bool ExtractTar(const char* inFileName, bool gzip, 
                          bool verbose);
   // This should be called first thing in main
   // it will keep child processes from inheriting the
@@ -438,6 +436,9 @@ public:
   static bool CheckRPath(std::string const& file,
                          std::string const& newRPath);
 
+  /** Remove a directory; repeat a few times in case of locked files.  */
+  static bool RepeatedRemoveDirectory(const char* dir);
+
 private:
   static bool s_ForceUnixPaths;
   static bool s_RunCommandHideConsole;
